using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TFSWrappers;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.Proxy;
using Microsoft.TeamFoundation.Build.Proxy;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using TabControl;
using Microsoft.Win32;
using Microsoft.TeamFoundation.Build.Client;
using UIComponents;

namespace TFSBuildManager
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            tcBuildTypes.Items.Clear();
            tsslServer.Text = String.Empty;
            Logon();
        }

        private void LoadBuildTypesTree()
        {
            tvBuildTypes.Nodes.Clear();
            string[] projectNames = serverWrapper.GetProjectNames();
            foreach (string projectName in projectNames)
            {
                TreeNode projectTreeNode = new TreeNode(projectName);
                projectTreeNode.ImageIndex = 0;
                projectTreeNode.SelectedImageIndex = 0;
                string[] buildTypeNames = buildStoreWrapper.GetBuildTypes(projectName);
                if (buildTypeNames.Length > 0)
                {
                    projectTreeNode.Nodes.Add("", "", -1);
                }
                tvBuildTypes.Nodes.Add(projectTreeNode);
            }
        }

        private void Logon()
        {
            string defaultServer = ConfigurationManager.AppSettings[Constants.DefaultServerAppSettingsKey];
            logFileViewer = ConfigurationManager.AppSettings[Constants.LogFileViewerAppSettingsKey];
            buildTypeFileViewer = ConfigurationManager.AppSettings[Constants.BuildTypeFileViewerAppSettingsKey];
            if ((defaultServer != null) && (defaultServer != String.Empty))
            {
                ProgressForm frmProgress = new ProgressForm();
                frmProgress.Show();
                Cursor = Cursors.WaitCursor;
                Application.DoEvents();
                try
                {
                    TFS = TFSFunctions.GetServer(defaultServer);
                }
                finally
                {
                    frmProgress.Close();
                    Cursor = Cursors.Default;
                }
            }
            else
            {
                SelectServer(true);
            }
        }

        private ServerWrapper serverWrapper;
        private SourceControlWrapper sourceControlWrapper;
        private WorkspaceWrapper workspaceWrapper;
        private BuildStoreWrapper buildStoreWrapper;
        private string logFileViewer;
        private string buildTypeFileViewer;

        private TeamFoundationServer tfs;

        public TeamFoundationServer TFS
        {
            get { return tfs; }
            set
            {
                tfs = value;
                if (tfs != null)
                {
                    tsslServer.Text = tfs.Name;
                    if (workspaceWrapper != null)
                    {
                        sourceControlWrapper.DeleteWorkspace(workspaceWrapper.WS.Name);
                    }
                    serverWrapper = new ServerWrapper(tfs);
                    sourceControlWrapper = new SourceControlWrapper(tfs);
                    workspaceWrapper = new WorkspaceWrapper(tfs, sourceControlWrapper.CreateTempWorkspace().Name);
                    buildStoreWrapper = new BuildStoreWrapper(tfs);
                    LoadBuildTypesTree();
                }
                else
                {
                    tsslServer.Text = String.Empty;
                }
            }
        }

        private void tscbGroupBy_SelectedIndexChanged(object sender, EventArgs e)
        {
            BaseTabControlItem baseItem = (BaseTabControlItem)tcBuildTypes.SelectedItem;
            if (baseItem != null)
            {
                if (baseItem.TabType == TabControlType.BuildType)
                {
                    BuildTypeTabControlItem item = (BuildTypeTabControlItem)tcBuildTypes.SelectedItem;
                    if (tscbGroupBy.Text == "Build Status")
                    {
                        item.GroupBy(BuildTypeGroupBy.BuildStatus);
                    }
                    else if (tscbGroupBy.Text == "Build Quality")
                    {
                        item.GroupBy(BuildTypeGroupBy.BuildQuality);
                    }
                    else if (tscbGroupBy.Text == "Complete Date")
                    {
                        item.GroupBy(BuildTypeGroupBy.CompleteDate);
                    }
                }
            }
       }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OptionsForm frmOptions = new OptionsForm(buildStoreWrapper);
            if (frmOptions.ShowDialog() == DialogResult.OK)
            {
                logFileViewer = frmOptions.LogFileViewer;
                buildTypeFileViewer = frmOptions.BuildTypeFileViewer;
                ConfigurationManager.RefreshSection("AppSettings"); 
            }
        }

        private void newBuildTypeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewBuildTypeForm frmNewBuildType = new NewBuildTypeForm();
            frmNewBuildType.ShowDialog();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutForm frmAbout = new AboutForm();
            frmAbout.ShowDialog();
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (workspaceWrapper != null)
            {
                sourceControlWrapper.DeleteWorkspace(workspaceWrapper.WS.Name);
            }
        }

        private void DeleteBuildType()
        {
            if (MessageBox.Show("Deleting a build type will remove the build type files from source control.\r\n" +
                "Are you sure that you want to you want to delete \"" + tvBuildTypes.SelectedNode.Text + "\" build type?", "TFSBuildManager", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                Cursor = Cursors.WaitCursor;
                try
                {
                    TFSBuildType buildType;
                    BuildTypeTabControlItem item = (BuildTypeTabControlItem)tcBuildTypes.Items.ItemByName(tvBuildTypes.SelectedNode.Text);
                    if (item != null)
                    {
                        buildType = item.TabBuildType;
                    }
                    else
                    {
                        buildType = new TFSBuildType(sourceControlWrapper, workspaceWrapper, buildStoreWrapper, tvBuildTypes.SelectedNode.Parent.Text, tvBuildTypes.SelectedNode.Text);
                    }
                    string buildTypeFolderServerPath = buildType.FileServerPath.Substring(0, 
                        buildType.FileServerPath.LastIndexOf('/'));
                    string buildTypeFolderLocalPath = Path.GetDirectoryName(buildType.FileLocalPath);
                    workspaceWrapper.Delete(buildTypeFolderServerPath, buildTypeFolderLocalPath, "Deleted by TFSBuildManager");
                    workspaceWrapper.ClearFileMapping(buildTypeFolderServerPath);
                    if (item != null)
                    {
                        tcBuildTypes.Items.Remove(item);
                    }
                    else
                    {
                        buildType.Dispose();
                        buildType = null;
                    }
                    tvBuildTypes.SelectedNode.Remove();
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error deleting build type. " + ex.Message, "TFSBuildManager",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                finally
                {
                    Cursor = Cursors.Default;
                }
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DeleteBuildType();
        }

        private void tvBuildTypes_AfterExpand(object sender, TreeViewEventArgs e)
        {
            if ((e.Node.Nodes.Count == 1) && (e.Node.Nodes[0].Text == ""))
            {
                Cursor = Cursors.WaitCursor;
                tvBuildTypes.BeginUpdate();
                try
                {
                    e.Node.Nodes.Clear();
                    TreeNode allBuildTypesTreeNode = new TreeNode(TFSWrappers.Constants.AllBuildTypes);
                    allBuildTypesTreeNode.ImageIndex = 2;
                    allBuildTypesTreeNode.SelectedImageIndex = 2;
                    e.Node.Nodes.Add(allBuildTypesTreeNode);
                    string[] buildTypeNames = buildStoreWrapper.GetBuildTypes(e.Node.Text);
                    foreach (string buildTypeName in buildTypeNames)
                    {
                        TreeNode buildTypeTreeNode = new TreeNode(buildTypeName);
                        buildTypeTreeNode.ImageIndex = 1;
                        buildTypeTreeNode.SelectedImageIndex = 1;
                        e.Node.Nodes.Add(buildTypeTreeNode);
                    }
                }
                finally
                {
                    tvBuildTypes.EndUpdate();
                    Cursor = Cursors.Default;
                }
            }
        }

        private BuildTypeTabControlItem CreateBuildTypeTab(string projectName, string buildTypeName)
        {
            Cursor = Cursors.WaitCursor;
            try
            {
                BuildTypeTabControlItem tabPage = new BuildTypeTabControlItem(sourceControlWrapper, 
                    workspaceWrapper, buildStoreWrapper, projectName, buildTypeName, 
                    tvBuildTypes.SelectedNode, buildTypeFileViewer, logFileViewer);
                tcBuildTypes.Items.Add(tabPage);
                tcBuildTypes.SelectedItem = tabPage;
                return tabPage;
            }
            finally
            {
                Cursor = Cursors.Default;
            }
        }

        private void OpenBuildType()
        {
            if (tvBuildTypes.SelectedNode.Level == 1)
            {
                TabControlItem item = tcBuildTypes.Items.ItemByName(tvBuildTypes.SelectedNode.Text.Replace(" ", "_"));
                if (item == null)
                {
                    BuildTypeTabControlItem tabItem = CreateBuildTypeTab(tvBuildTypes.SelectedNode.Parent.Text, tvBuildTypes.SelectedNode.Text);
                    tvBuildTypes.SelectedNode.Tag = tabItem.TabBuildType;
                }
                else
                {
                    tcBuildTypes.SelectedItem = item;
                }
            }
        }

        private void tvBuildTypes_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            OpenBuildType();
        }

        private void SelectServer(bool showProgress)
        {
            DomainProjectPicker domainProjectPicker = new DomainProjectPicker(DomainProjectPickerMode.None);
            if (domainProjectPicker.ShowDialog() == DialogResult.OK)
            {
                ProgressForm frmProgress = new ProgressForm();
                if (showProgress)
                {
                    frmProgress.Show();
                }
                Cursor = Cursors.WaitCursor;
                Application.DoEvents();
                try
                {
                    TFS = domainProjectPicker.SelectedServer;
                }
                finally
                {
                    if (showProgress)
                    {
                        frmProgress.Close();
                    }
                    Cursor = Cursors.Default;
                }
            }
        }

        private void selectServerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SelectServer(false);
        }

        private void tsbSelectProject_Click(object sender, EventArgs e)
        {
            SelectServer(false);
        }

        private void tvBuildTypes_AfterSelect(object sender, TreeViewEventArgs e)
        {
            bool isLevel1 = e.Node.Level == 1;
            bool notAllBuildTypes = e.Node.Text != TFSWrappers.Constants.AllBuildTypes;
            openToolStripMenuItem1.Enabled = isLevel1;
            tsbOpenBuildType.Enabled = isLevel1;
            openToolStripMenuItem2.Enabled = isLevel1;
            deleteToolStripMenuItem1.Enabled = isLevel1 && notAllBuildTypes;
            deleteToolStripMenuItem2.Enabled = isLevel1 && notAllBuildTypes;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenBuildType();
        }

        private void tsbOpenBuildType_Click(object sender, EventArgs e)
        {
            OpenBuildType();
        }

        private void tvBuildTypes_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                tvBuildTypes.SelectedNode = e.Node;
            }
        }

        private void tcBuildTypes_TabControlItemSelectionChanged(TabControlItemChangedEventArgs e)
        {
            tscbGroupBy_SelectedIndexChanged(null, null);
        }
    }

    public enum BuildTypeGroupBy
    {
        BuildStatus,
        BuildQuality,
        CompleteDate
    }
}